% -------------------------------------------------------------------------
% Program: simple CRS/VRS input-oriented DEA problem
% Author:  Eugene P. Morgunov 
% Date:    May 22, 2005
% -------------------------------------------------------------------------

% This example of solving CRS input-oriented DEA problem is taken
% from the file deap.pdf (p. 15) and extended for VRS case

% we have 6 variables -- 1 variable is theta
%                        5 variables are lambda's
% we name all them as x's (for simplicity) --
% so x( 1 ) == theta
%    x( 2 ) == lambda1
%    x( 3 ) == lambda2, etc.

clear;

% scale assumption
% scale = 'VRS';
scale = 'CRS';

fprintf( 'Scale assumption is %s\n\n', scale );

% objective function
% only coefficient by theta is 1 and coefficients by lambda's are zeros
f = [ 1; 0; 0; 0; 0; 0 ];

% this fragment can be deleted
% matrix of constraints is formed for DMU 3
% A = [  0 -1 -2 -3 -1 -2;
%       -6  2  2  6  3  6;
%       -6  5  4  6  2  2
%     ];

X_orig = [ 2 2 6 3 6;
           5 4 6 2 2
         ];

Y_orig = [ 1 2 3 1 2 ];

% we make all elements of this vector to be negative because of type
% of constraint for Y's (outputs) -- this constraint is '>=', but we need
% '<=' for using linprog(). So we muptiply both parts of the constraint by -1
% and invert '>=' for '<='
Y_orig_negative = Y_orig * (-1);

% lower bounds on decision variables
lb = zeros( 6, 1 );

% if scale assumption is VRS then we need one more constraint --
% for lambda's (their sum equals to 1)
if scale == 'VRS'
  % zero at the first position -- because coeff. by theta in the constraint
  % for lambda's is zero
  Aeq = [ 0 1 1 1 1 1 ];
  beq = [ 1 ];            % sum of lambda's equals to 1
end

% matrix of results
results = [];

for i = 1 : 5      % we have 5 DMUs
  fprintf( 'DMU number %d\n', i );
  fprintf( '---------------------\n\n' );

  % matrix of constraints is formed for every DMU

  % we begin with matrix of outputs (y's)
  Y = [ 0                      Y_orig_negative ];

  % the first column in X matrix is x's, multiplied by -1,
  % of current (being evaluated) DMU
  X = [ X_orig( :, i ) * (-1)  X_orig ];
  
  % now we combine constraints for y's and x's into matrix A
  A = [ Y; X ];

  % column of RHSs
  % constraint for Y's will have the value of y for evaluated DMU,
  % multiplied by -1, as RHS
  % RHSs for x's are zeros
  b = [ Y_orig( i ) * (-1); zeros( 2, 1 ) ];
  
  % call to linprog() function for solving linear problem
  if scale == 'CRS'
    % we don't have constraints in the form of equalities so pass empty matrices
    x = linprog( f, A, b, [], [], lb );       % CRS
  elseif scale == 'VRS'
    x = linprog( f, A, b, Aeq, beq, lb );     % VRS
  end

  disp( x );

  % the value of efficiency measure that current (being evaluated) DMU has
  fprintf( 'Value of theta = %f\n\n', x( 1, 1 ) );

  % accumulate results
  results = [ results   x( 1, 1 ) ];
  
  % reference DMU's and their weights
  for j = 2 : 6   % lambda's have numbers from 2 to 6
    if ( x( j ) > 0.000001 )  % we use 0.000001 instead of zero
      % (j - 1) -- because the first lambda has number 2, etc.
      % fprintf( 'DMU %d   Weight = %20.18f\n', j - 1, x( j ) );
      fprintf( 'DMU %d   Weight = %5.4f\n', j - 1, x( j ) );
    end
  end
  fprintf( '---------------------\n\n' );
end

% print all efficiencies together
for i = 1 : length( results )
  fprintf( 'DMU %d has efficiency %f\n', i, results( i ) );
end
  