% -------------------------------------------------------------------------
% Program: simple CRS/VRS input-oriented DEA problem with support of
%          reading data from a text file
% Author:  Eugene P. Morgunov 
% Date:    May 22, 2005
% -------------------------------------------------------------------------

% NOTE. DEA problem is in ENVELOPMENT form (see Coelli)

% numbering of variables is as follows--
%    x( 1 ) == theta
%    x( 2 ) == lambda1
%    x( 3 ) == lambda2, etc.

clear;

% ask a user to enter name of instruction file
instruct_file = input( 'Enter instruction file name: ', 's' );

% open instruction file
ins = fopen( instruct_file, 'r' );

if ( ins == -1 )
  fprintf( 'Can not open instruction file %s', instruct_file );
  return;
end

% now read instruction file line by line
% this file must have rigid structure (note that all words in comments
% that are contained in instruction file must be connected with
% underscores '_')

% get data file name
line = fgetl( ins );
data_file = sscanf( line, '%s %*s' );

% get output file name
line = fgetl( ins );
output_file = sscanf( line, '%s %*s' );

% get scale assumption
line = fgetl( ins );
scale = sscanf( line, '%s %*s' );

% get number of DMUs
line = fgetl( ins );
DMUs_num = sscanf( line, '%d ' );

% get number of outputs
line = fgetl( ins );
outputs_num = sscanf( line, '%d ' );

% get number of inputs
line = fgetl( ins );
inputs_num = sscanf( line, '%d ' );

% instruction file is not necessary any more
fclose( ins );

fprintf( 'Scale assumption is %s\n\n', scale );

% now we can get all data from datafile
% function get_data() is placed in separate file get_data.m
[ X_orig, Y_orig ] = get_data( data_file, DMUs_num, inputs_num, outputs_num );


% objective function
% only coefficient by theta is 1 and coefficients by lambda's are zeros
f = [ 1; zeros( DMUs_num, 1 ) ];

% we make all elements of this vector to be negative because of type
% of constraint for Y's (outputs) -- this constraint is '>=', but we need
% '<=' for using linprog(). So we muptiply both parts of the constraint by -1
% and invert '>=' for '<='
Y_orig_negative = Y_orig * (-1);

% lower bounds on decision variables
lb = zeros( DMUs_num + 1, 1 );

% if scale assumption is VRS then we need one more constraint --
% for lambda's (their sum equals to 1)
if scale == 'VRS'
  % zero at the first position -- because coeff. by theta in the constraint
  % for lambda's is zero
  Aeq = [ 0, ones( 1, DMUs_num ) ];
  beq = [ 1 ];            % sum of lambda's equals to 1
end

% matrix of results
results = [];

% open output file for writing
out = fopen( output_file, 'w' );

if ( out == -1 )
  fprintf( 'Can not open output file %s', output_file );
  return;
end

% now we loop through all DMUs
for i = 1 : DMUs_num
  fprintf( out, 'DMU number %d\n', i );
  fprintf( out, '---------------------\n\n' );

  % matrix of constraints is formed for every DMU

  % we begin with matrix of outputs (y's)
  Y = [ zeros( outputs_num, 1 )  Y_orig_negative ];

  % the first column in X matrix is x's, multiplied by -1,
  % of current (being evaluated) DMU
  X = [ X_orig( :, i ) * (-1)  X_orig ];
  
  % now we combine constraints for y's and x's into matrix A
  A = [ Y; X ];

  % column of RHSs
  % constraint for Y's will have the value of y for evaluated DMU,
  % multiplied by -1, as RHS
  % RHSs for x's are zeros
  b = [ Y_orig( :, i ) * (-1); zeros( inputs_num, 1 ) ];
  
  % call to linprog() function for solving linear problem
  if scale == 'CRS'
    % we don't have constraints in the form of equalities so pass empty matrices
    x = linprog( f, A, b, [], [], lb );       % CRS
  elseif scale == 'VRS'
    x = linprog( f, A, b, Aeq, beq, lb );     % VRS
  end

  % display results of linprog() function call
  disp( x );

  % the value of efficiency measure that current (being evaluated) DMU has
  fprintf( out, 'Efficiency measure (Value of theta) = %f\n\n', x( 1, 1 ) );

  % accumulate results
  results = [ results   x( 1, 1 ) ];
  
  % reference DMU's and their weights
  for j = 2 : DMUs_num + 1  % lambda's have numbers from 2 to DMUs_num + 1
    if ( x( j ) > 0.000001 )  % we use 0.000001 instead of zero
      % (j - 1) -- because the first lambda has number 2, etc.
      fprintf( out, 'DMU %d   Weight = %5.4f\n', j - 1, x( j ) );
    end
  end
  fprintf( out, '---------------------\n\n' );
end

% print all efficiencies together
for i = 1 : length( results )
  fprintf( out, 'DMU %d has efficiency %f\n', i, results( i ) );
end

% close output file
fclose( out );
